#version 120

/*
BlazingPack.pl MotionBlur

Inspired by many similar shader packs.
This file actually fixes the hand ghosting bug.

Optimized by gamma correction 2.0 instead of 2.2.

*/	

const float intensity = 0.000777;

varying vec4 texture;


uniform sampler2D depthtex0;
uniform sampler2D colortex0;
uniform sampler2D colortex7;

uniform mat4 gbufferProjection;
uniform mat4 gbufferProjectionInverse;
uniform mat4 gbufferPreviousProjection;
uniform mat4 gbufferModelViewInverse;
uniform mat4 gbufferPreviousModelView;

uniform vec3 cameraPosition;
uniform vec3 previousCameraPosition;

uniform float frameTime;
uniform float viewWidth;
uniform float viewHeight;


bool isHandAt(vec2 tex){
    vec3 mat = texture2D(colortex7, tex).rgb;
    return mat.r > 0.97 && mat.r < 0.99 && mat.g > 0.97 && mat.g < 0.99;
}
void main() {
    vec2 coord = texture.st;
    float depth = texture2D(depthtex0, coord.st).x;
	vec2 pixel = 2.0/vec2(viewWidth, viewHeight);

	vec4 currentPosition = vec4(coord.x * 2.0 - 1.0, coord.y * 2.0 - 1.0, 2.0 * depth - 1.0, 1.0);
	
	vec4 fragposition = gbufferProjectionInverse * currentPosition;
	fragposition = gbufferModelViewInverse * fragposition;
	fragposition /= fragposition.w;
	fragposition.xyz += cameraPosition;
	
	vec4 previousPosition = fragposition;
	previousPosition.xyz -= previousCameraPosition;
	previousPosition = gbufferPreviousModelView * previousPosition;
	previousPosition = gbufferPreviousProjection * previousPosition;
	previousPosition /= previousPosition.w;


	vec2 velocity = (currentPosition - previousPosition).xy;
	velocity = velocity/(1.0+length(velocity))*intensity/frameTime;

    vec3 color = vec3(0.0);
    float countSamples = 1.0;
    
	vec2 iteratorCoord = coord.st-velocity*5.0;
    if (!isHandAt(coord.st)){
        for (int i = 1; i < 11; ++i) {
                vec2 coordScreenFix = clamp(iteratorCoord, pixel, 1.0-pixel);
                if(!isHandAt(coordScreenFix)){
                    vec3 colorNearby = texture2D(colortex0, coordScreenFix).rgb;
                    color += colorNearby*colorNearby;
                    countSamples += 1.0;
                }
            
            iteratorCoord += velocity;
        }
        color /= countSamples;
        color = sqrt(color);
    }else{
        color = texture2D(colortex0, coord.st).rgb;
    }
    
    /* DRAWBUFFERS:0 */
    gl_FragData[0] = vec4(color,1.0);
}







